package RPC::OGo;
#
# $Id: OGo.pm,v 1.2 2003/12/15 14:39:46 martin Exp $
#

$VERSION = "0.1.1";


use Carp;
use XMLRPC::Lite;

use strict;
use vars qw($DEBUG);
use RPC::OGo::Date;

$ENV{'DEBUG'} ? $DEBUG = $ENV{'DEBUG'} : $DEBUG = 0;

=head1 NAME RPC::OGo - Access OpenGroupware.org data remote with xmlrpc

=head1 SYNOPSIS

    use Ogo;
    my $ogo = new RPC::OGo("rootpassword");


=head1 DESCRIPTION

The RPC::OGo module is the RPC::OGo base class. It handles the access to the
OpenGroupware.org XMLRPC Server.

=head1 METHODS

=head2 RPC::OGo->new(<$password> || <\%settings>)

The constructor new() creates a new RPC::OGo object. This object will
handle the connection to the XML-RPC Server. The constructor could be
called in two ways. If only one parameter is given, this will be used
as the password to connect the XML-RPC daemon as root on port 20001 at
localhost.

Alternatively one can call it with the following settings

   password => use this password to connect (required)
   login    => use this login to connect (optional)
   host     => connect to this host (optional)
   port     => connect to this port (optional)

=cut
sub new {
    RPC::OGo::reconnect(@_);
}

=head2 reconnect(<$password> || <\%settings>)

(Re)connect to the XML-RPC server with the given settings.

=cut
sub reconnect {
    my $class = shift;
    my @param = @_;

    my $self = { error => 0 };
    if (ref $param[0]) {
        $self = { login     => $param[0]{'login'}    || 'root',
                  password  => $param[0]{'password'} || 'password',
                  host      => $param[0]{'host'}     || 'localhost',
                  port      => $param[0]{'port'}     || '20001',
                };
    }
    else {
        $self = { password  => $param[0],
                  login     => 'root',
                  host      => 'localhost',
                  port      => '20001',
                };
    }
    bless $self, $class;

    my $user = $self->{'login'};
    my $host = $self->{'host'};
    my $port = $self->{'port'};
    my $pass = $self->{'password'};

    unless (defined $RPC::OGo->{'rpc'}) {
        croak "Missing RPC::OGo password\n" unless $self->{password};
        my $connect = XMLRPC::Lite
            ->proxy("http://$user:$pass\@$host:$port/x/xmlrpc");
        $self->{'rpc'} = $connect;
        $RPC::OGo = $self;
    }
    return $RPC::OGo;
}


=head2 call($package,$method,@params)

This method will execute arbitrary OGo methods via XML-RPC. The
parameter $domain represents the OGo package (see XML-RPC
documentation) on which $method will called with the params in
@params.

The response of the XML-RPC call will returned. To create an object of
the response, one should use the method "createObject".

=cut
sub call {
    my ($self,$package,$method,@params) = @_;

    my $nspace = lc($package);
    my $callfunc = $nspace . '.' . $method;

    my $som = $RPC::OGo->{'rpc'}->call($callfunc,@params);
    if ($som->fault) {
        carp "XMLRPC error:\n  ",$som->faultstring,"\n";
        $self->{'error'} = 99;
        return undef;
    }
    my $res = $som->result;
    $self->{'error'} = 10 unless $res;
    return $res;
}


=head2 _call($method,@params)

This private method will be inherited from the subclasses. I calls
$method with the on the corrrect XMLRPC RPC::OGo object and should
only be used internal by RPC::OGo classes.

=cut
sub _call {
    my ($self,$method,@params) = @_;

    my ($package) = ref($self) =~ /^\S+::(\S+)$/;

    return $self->call($package,$method,@params);
}


=head2 connection()

Returns the current RPC::OGo connection object which is an XMLRPC::Lite
object.

=cut
sub connection { return shift()->{'_connect'}; }


########################################################################
# common methods
########################################################################

=head2 fetchIds([$qualifier,[$strip]])

Fetches the Ids of all person with name match to the $qualifier. The
qualifier can contain a asterisk charakter for wildcards. If $strip is
a true value, the plain id will returned instead of the URI form. The
XMLRPC API accepts both forms.

 Returns the list of Ids.

=cut
sub fetchIds {
    my ($self,$qualifier,$strip) = @_;

    my $fattr = "name";
    my $ret = $self->_call("fetchIds",
                      { qualifier => "$fattr like \'$qualifier\'"});

    if ($strip) {
        foreach my $id (@{$ret}) {
            # strip ids from string
            $id =~ s/^.+?([^\/]+)$/$1/;
        }
    }
    return @{$ret};
}


=head2 deleteByNumber($number)

Deletes the object with the number $number. Returns the deleted
object.

=cut
sub deleteByNumber {
    my ($self,$attr) = @_;

    $self->_warn('number') unless $attr;
    my $ret = $self->_call('deleteByNumber',$attr);

    $self->{'error'} = 0; # no return value from XMLRPC call
    return RPC::OGo::Document->createObject($self,$ret);
}


=head2 getById([$id])

Fetches the document of the person with the id $id. Returns the
RPC::OGo::Person object.

=cut
sub getById {
    my ($self,$id) = @_;

    my $ret = $self->_call("getById",$id);

    # return the assigned document
    return RPC::OGo::Document->createObject($self,$ret);
}


=head2 insert([$document])

Inserts the current object into the OGo database. The optional hashref
or object $params could set or overwrite some of the attributes of the
current object. Returns assigned document returned from OGo.

=cut
sub insert {
    my ($self,$params) = @_;
    if (defined $params) {
        foreach (keys %{$params}) {
            $self->{$_} = $params->{$_};
        }
    }
    my $ret = $self->_call('insert',$self);

    return RPC::OGo::Document->createObject($self,$ret);
}


=head2 update([$document])

Updates the current object at the OGo database. The optional hashref
or object $params could set or overwrite some of the attributes of the
current object. Returns the updated document returned from OGo.

=cut
sub update {
    my ($self,$params) = @_;
    if (defined $params) {
        foreach (keys %{$params}) {
            $self->{$_} = $params->{$_};
        }
    }
    my $ret = $self->_call('update',$self);

    return RPC::OGo::Document->createObject($self,$ret);
}





########################################################################
### private methods ###
# error warnings
sub _warn {
    my ($self,$name) = @_;
    carp "Attribute $name not defined";
}


### EOP ###
1;
__END__

=head1 ATTRIBUTES

=head2 error

The error attribute contains the error number. It will be set by some
methods. Error numbers are:

   0  - o.k.
  10  - XML-RPC call returns nothing.
  99  - XML-RPC remote error.

=head1 AUTHOR

Martin Werthmoeller <lt>mw@werthmoeller.de<gt>

=head1 COPYRIGHT

Copyright (C) 2003  Martin Werthmoeller

This library is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=head1 SEE ALSO

L<perl>, L<RPC::OGo::Document>.

=cut
