package RPC::OGo::Document;
#
# $Id: Document.pm,v 1.2 2003/12/15 14:39:46 martin Exp $
#

$VERSION = "0.1";

use Carp;
use strict;
use vars qw/$AUTOLOAD $DESTROY/;


=head1 NAME

RPC::OGo::Document - Base Document class

=head1 SYNOPSIS

my $id = RPC::OGo::Document::<Domain>->id;

=head1 DESCRIPTION

Base class for all RPC::OGo::Document::* classes.

=head1 METHODS

=cut


=head2 <method>([$value])

Sets the attribute which has the same name as <method> to $value if
$value is set. Returns the value of attribute <method>.

=cut
# universal method to attribute mapping.
sub AUTOLOAD {
    my ($self,$value) = shift;

    # $1 Class, $2 method name
    my ($class,$attr)= $AUTOLOAD =~ /^(.*)::(\w+)/;

    unless (exists $self->{$attr}) {
        $self->_warn($attr);
        return '';
    }
    if (defined ($value)) {
        $self->{$attr} = $value
    }
    return $self->{$attr};
}


=head2 setAttr(<$attribute,$value>)

Sets the value of the attribute with name $attribute. You can find the
description of the attribute names at the OpenGroupware XMLRPC
documentation. With this method you will set the attributes at the
first level.

To set attributes in nested data structures you should prepare the
substructure (mostly a hashref) and set this at value.

The method returns the RPC::OGo::Document object or an empty scalar.

=cut
sub setAttr {
    my ($self,$attr,$value) = @_;
    unless (defined ($value)) {
        $self->_warn("value");
        return '';
    }
    unless (exists $self->{$attr}) {
        $self->_warn($attr);
        return '';
    }
    $self->{$attr} = $value;
    return $self;
}


=head2 createObject($caller,$data)

Creates an object of RPC::OGo::Document::$caller with the $data data
structure. $caller is the calling object. Example how to call this
method:

  my $document = RPC::OGo::Document::createObject($self,$data);

Only internal used. Returns the RPC::OGo::Document::<subclass> object.

=cut
sub createObject {
    my ($self,$caller,$data) = @_;

    my ($subclass) = ref($caller)  =~ /^\S+::(\S+)$/;
    my $class = "RPC::OGo::Document::".$subclass;
    my $obj = $class->new($data);

    return $obj;
}



########################################################################
# often used data structures
########################################################################

=head2 fax()

This method returns the fax number.

=cut
sub fax {
    my ($self) = @_;
    if (defined ($self->{'phones'}->{'10_fax'}->{'number'})) {
        return$ self->{'phones'}->{'10_fax'}->{'number'};
    }
    else {
        return '';
    }
}


=head2 phone()

This method returns the first occurance of a phone number at the
document structure.

=cut
sub phone {
    my ($self) = @_;

    return '' unless (exists $self->{'phones'});

    if (defined ($self->{'phones'}->{'01_tel'}->{'number'})) {
        return$ self->{'phones'}->{'01_tel'}->{'number'};
    }
    elsif (defined ($self->{'phones'}->{'02_tel'}->{'number'})) {
        return$ self->{'phones'}->{'02_tel'}->{'number'};
    }
    elsif (defined ($self->{'phones'}->{'05_tel_private'}->{'number'})) {
        return$ self->{'phones'}->{'05_tel_private'}->{'number'};
    }
    elsif (defined ($self->{'phones'}->{'03_tel_funk'}->{'number'})) {
        return$ self->{'phones'}->{'03_tel_funk'}->{'number'};
    }
    return '';
}


=head2 phone1(), phone2()

This method returns the value of phoneN where N is one of 1-n defined
phone numbers at the data structure.

=cut
sub phone1 {
    my ($self) = @_;
    if (defined ($self->{'phones'}->{'01_tel'}->{'number'})) {
        return$ self->{'phones'}->{'01_tel'}->{'number'};
    }
    else {
        return '';
    }
}

sub phone2 {
    my ($self) = @_;
    if (defined ($self->{'phones'}->{'02_tel'}->{'number'})) {
        return$ self->{'phones'}->{'02_tel'}->{'number'};
    }
    else {
        return '';
    }
}




# don't call AUTOLOAD as destroy
sub DESTROY {}

# error warnings
sub _warn {
    my ($self,$name) = @_;
    carp "Attribute \"$name\" not defined";
}

### EOP ###
1;
__END__

=head1 AUTHOR

Martin Werthmoeller <lt>mw@werthmoeller.de<gt>

=head1 SEE ALSO

L<perl>, L<RPC::OGo>,
L<RPC::OGo::Account>,L<RPC::OGo::Person>,L<RPC::OGo::Enterprise>.

=cut
